<?php
/**
 * The FNMM_Public class handles the public-facing functionality of the Fly Nav Mobile plugin.
 */
class FNMM_Public {
    /**
     * Constructor method.
     * Adds action hooks for enqueueing styles and scripts.
     */
    public function __construct() {
        add_action('wp_enqueue_scripts', array($this, 'enqueue_styles'));
        add_action('wp_enqueue_scripts', array($this, 'enqueue_scripts'));
        // Add new action for preloading logo
        add_action('wp_head', array($this, 'preload_logo'), 1);
    }

    /**
     * Preload logo image
     */
    public function preload_logo() {
        if (!wp_is_mobile()) {
            return;
        }
        
        $options = get_option('fnmm_settings');
        $logo_url = isset($options['logo_url']) ? $options['logo_url'] : '';
        
        if ($logo_url) {
            echo '<link rel="preload" href="' . esc_url($logo_url) . '" as="image">';
        }
    }

    /**
     * Enqueue styles for the public-facing side of the site.
     */
    public function enqueue_styles() {
        // Only enqueue if we're not in the admin area and the device is mobile
        if (!is_admin() && wp_is_mobile()) {
            wp_enqueue_style('fly-nav-mobile', plugin_dir_url(__FILE__) . 'css/fly-nav-mobile-public.css', array(), FNMM_VERSION, 'all');

            // Get plugin settings
            $options = get_option('fnmm_settings');
            // Generate custom CSS based on user settings
            $custom_css = $this->generate_custom_css($options);
            // Add inline custom CSS
            wp_add_inline_style('fly-nav-mobile', esc_html($custom_css));

            // Add user's custom CSS if any
            $user_custom_css = get_option('fnmm_custom_css', '');
            if (!empty($user_custom_css)) {
                wp_add_inline_style('fly-nav-mobile', esc_html($user_custom_css));
            }
        }
    }

    /**
     * Enqueue scripts for the public-facing side of the site.
     * Currently not in use, but prepared for future JavaScript additions.
     */
    public function enqueue_scripts() {
        // Remove this method if it's empty, or leave it blank if you need it for future use
    }

    /**
     * Render the mobile menu HTML structure.
     */
    public function render_mobile_menu() {
        // Only render the mobile menu if the device is mobile
        if (!wp_is_mobile()) {
            return;
        }

        // Get plugin settings
        $options = get_option('fnmm_settings');
        $logo_url = isset($options['logo_url']) ? $options['logo_url'] : '';
        $menu_id = isset($options['selected_menu']) ? $options['selected_menu'] : '';

        // Start output buffering
        ob_start();
        ?>
        <div id="fly-nav-mobile-menu">
            <!-- Hidden Checkbox for Toggling Menu -->
            <input type="checkbox" id="fnmm-menu-toggle" class="fnmm-toggle menu-toggle">
            
            <!-- Hidden Checkbox for Toggling Search -->
            <input type="checkbox" id="fnmm-search-toggle" class="fnmm-toggle search-toggle">

            <div class="fnmm-header">
                <div class="fnmm-logo">
                    <?php if ($logo_url): 
                        // Get attachment ID from URL
                        $attachment_id = attachment_url_to_postid($logo_url);
                        if ($attachment_id) {
                            // Get alt text from attachment
                            $image_alt = get_post_meta($attachment_id, '_wp_attachment_image_alt', true);
                            $alt_text = $image_alt ? $image_alt : get_bloginfo('name');
                        } else {
                            // Use site name as alt text if no attachment found
                            $alt_text = get_bloginfo('name');
                        }
                    ?>
                        <a href="<?php echo esc_url(home_url('/')); ?>" title="<?php echo esc_attr(get_bloginfo('name')); ?>">
                            <img src="<?php echo esc_url($logo_url); ?>" alt="<?php echo esc_attr($alt_text); ?>" fetchpriority="high" loading="eager" decoding="async">
                        </a>
                    <?php else: ?>
                        <a href="<?php echo esc_url(home_url('/')); ?>" title="<?php echo esc_attr(get_bloginfo('name')); ?>">
                            <h1><?php echo esc_html(get_bloginfo('name')); ?></h1>
                        </a>
                    <?php endif; ?>
                </div>
                <div class="fnmm-icons">
                    <!-- New Search Icon -->
                    <?php 
                    $search_option = isset($options['search_icon']) ? $options['search_icon'] : 'show';

                    if ($search_option !== 'hide'): ?>
                        <div class="search-icon-wrapper">
                            <label for="fnmm-search-toggle" class="search-icon" aria-label="<?php esc_attr_e('Toggle Search', 'fly-nav-mobile'); ?>">
                                <div class="circle"></div>
                                <div class="handle"></div>
                            </label>
                        </div>
                    <?php endif; ?>
                    <!-- Hamburger Menu Icon -->
                    <label for="fnmm-menu-toggle" class="hamburger-icon" aria-label="<?php esc_attr_e('Toggle Menu', 'fly-nav-mobile'); ?>">
                        <div class="line"></div>
                        <div class="line"></div>
                        <div class="line"></div>
                    </label>
                </div>
            </div>
            <nav class="fnmm-menu">
                <?php
                // Display the selected menu if a menu ID is set
                if ($menu_id) {
                    wp_nav_menu(array(
                        'menu' => intval($menu_id),
                        'container' => false,
                        'menu_class' => 'fnmm-menu-items',
                        'fallback_cb' => false,
                        'depth' => 2, // Allow one level of submenu
                        'walker' => new FNMM_Walker_Nav_Menu(),
                    ));
                }
                ?>
            </nav>
            <div class="fnmm-search-form">
                <form role="search" method="get" class="search-form" action="<?php echo esc_url(home_url('/')); ?>">
                    <label>
                        <span class="screen-reader-text"><?php echo _x('Search for:', 'label', 'fly-nav-mobile'); ?></span>
                        <input type="search" class="fnmm-search-field" placeholder="<?php echo esc_attr_x('Search &hellip;', 'placeholder', 'fly-nav-mobile'); ?>" value="<?php echo get_search_query(); ?>" name="s" />
                    </label>
                    <button type="submit" class="fnmm-search-submit">
                        <?php echo esc_html_x('Search', 'submit button', 'fly-nav-mobile'); ?>
                    </button>
                </form>
            </div>
        </div>
        <?php
        // Capture the buffered content
        $output = ob_get_clean();
        
        // Output the content using wp_kses to allow only specific HTML tags
        echo wp_kses($output, array(
            'div' => array('id' => array(), 'class' => array()),
            'input' => array('type' => array(), 'id' => array(), 'class' => array(), 'placeholder' => array(), 'value' => array(), 'name' => array()),
            'img' => array('src' => array(), 'alt' => array(), 'fetchpriority' => array(), 'loading' => array()),
            'h1' => array(),
            'label' => array('for' => array(), 'class' => array(), 'aria-label' => array()),
            'nav' => array('class' => array()),
            'ul' => array('class' => array()),
            'li' => array('class' => array()),
            'a' => array('href' => array(), 'class' => array()),
            'form' => array('role' => array(), 'method' => array(), 'class' => array(), 'action' => array()),
            'span' => array('class' => array()),
            'button' => array('type' => array(), 'class' => array()),
            'svg' => array('class' => array(), 'aria-hidden' => array(), 'role' => array(), 'focusable' => array(), 'xmlns' => array(), 'width' => array(), 'height' => array(), 'viewBox' => array()),
            'path' => array('d' => array()),
        ));
    }

    /**
     * Generate custom CSS based on user settings.
     *
     * @param array $options User-defined plugin settings.
     * @return string Custom CSS.
     */
    private function generate_custom_css($options) {
        // Initialize hide header class
        $hide_header_class = '';
        if (isset($options['hide_header_class']) && !empty($options['hide_header_class'])) {
            // Ensure the class or ID selector is properly formatted
            if ($options['hide_header_class'][0] !== '.' && $options['hide_header_class'][0] !== '#') {
                // Default to class if no selector is provided
                $hide_header_class = '.' . $options['hide_header_class'];
            } else {
                $hide_header_class = $options['hide_header_class'];
            }
        }

        // Get icon color from options or use default
        $icon_color = isset($options['icon_color']) ? esc_attr($options['icon_color']) : '#333';

        // Get menu font color from options or use default
        $menu_font_color = isset($options['menu_font_color']) ? esc_attr($options['menu_font_color']) : '#000000';

        // Update the :root CSS variables
        $custom_css = "
            :root {
                --fnmm-header-bg: " . (isset($options['header_bg_color']) ? esc_attr($options['header_bg_color']) : '#ffffff') . ";
                --fnmm-dropdown-bg: " . (isset($options['dropdown_bg_color']) ? esc_attr($options['dropdown_bg_color']) : '#ffffff') . ";
                --fnmm-font-family: " . (isset($options['menu_font']) ? esc_attr($options['menu_font']) : 'Arial') . ", sans-serif;
                --fnmm-font-size: " . (isset($options['font_size']) ? esc_attr($options['font_size']) : '16') . "px;
                --fnmm-logo-width: " . (isset($options['logo_width']) ? esc_attr($options['logo_width']) : '100') . "px;
                --fnmm-menu-font-color: {$menu_font_color};
            }
            @media (max-width: 768px) {
                /* Mobile menu styles */
                #fly-nav-mobile-menu {display:block;position:absolute;top:0;left:0;width:100%;z-index:9999}
                " . esc_html($hide_header_class) . " {display:none!important}
                .fnmm-header {position:relative;width:100%;background-color:var(--fnmm-header-bg);box-shadow:0 2px 5px rgba(0,0,0,.1)}
                #fly-nav-mobile-menu .fnmm-menu {background-color:var(--fnmm-dropdown-bg)}
                #fly-nav-mobile-menu .fnmm-menu-items {font-family:var(--fnmm-font-family);font-size:var(--fnmm-font-size)}
                #fly-nav-mobile-menu .fnmm-logo img {max-width:var(--fnmm-logo-width)}
                .fnmm-toggle {position:absolute;opacity:0;width:0;height:0}
                .fnmm-menu {display:none;background-color:#fff;padding:10px;box-shadow:0 2px 5px rgba(0,0,0,.1);position:absolute;top:60px;right:20px;width:90%;max-width:300px;z-index:10000;border-radius:4px}
                #fnmm-menu-toggle:checked~.fnmm-menu {display:block}
                .fnmm-menu-items {list-style-type:none;padding:0;margin:0}
                .fnmm-menu-items li {padding:10px;border-bottom:1px solid rgba(0,0,0,.1)}
                .fnmm-menu-items li:last-child {border-bottom:none}
                .fnmm-menu-items .sub-menu {display:none;padding-left:20px}
                .fnmm-submenu-toggle {
                    display: none;
                }
                .fnmm-submenu-label {
                    display: inline-block;
                    width: 30px;
                    height: 30px;
                    position: absolute;
                    right: 0;
                    top: 5px;
                    cursor: pointer;
                    text-align: center;
                    line-height: 30px;
                }
                .arrow-down-symbol {
                    width: 0;
                    height: 0;
                    border-left: 8px solid transparent;
                    border-right: 8px solid transparent;
                    border-top: 9px solid var(--fnmm-menu-font-color);  /* Use the menu font color for the arrow */
                    display: inline-block;
                    transition: transform 0.3s ease;
                }
                .fnmm-menu-items .menu-item-has-children {
                    position: relative;
                }
                .fnmm-menu-items .menu-item-has-children > a {
                    padding-right: 40px;
                }
                .fnmm-menu-items .sub-menu {
                    display: none;
                }
                .fnmm-submenu-toggle:checked ~ .sub-menu {
                    display: block;
                }
                .fnmm-submenu-toggle:checked + .fnmm-submenu-label .arrow-down-symbol {
                    transform: rotate(180deg);
                }

                /* Search icon styles */
                .search-icon-wrapper {
                    display: flex;
                    justify-content: center;
                    align-items: center;
                }

                .fnmm-search-toggle {
                    display: none;
                }

                .search-icon {
                    display: flex;
                    justify-content: center;
                    align-items: center;
                    width: 30px;
                    height: 30px;
                    cursor: pointer;
                    position: relative;
                    transition: all 0.3s ease-in-out;
                }

                .circle {
                    width: 20px;
                    height: 20px;
                    border: 3px solid {$icon_color};
                    border-radius: 50%;
                    transition: all 0.3s ease-in-out;
                }

                .handle {
                    position: absolute;
                    width: 12px;
                    height: 3px;
                    background-color: {$icon_color};
                    border-radius: 1.5px;
                    transform-origin: left center;
                    right: -3px;
                    bottom: 7px;
                    transition: all 0.3s ease-in-out;
                    transform: rotate(45deg);
                }

                #fnmm-search-toggle:checked ~ .fnmm-header .search-icon .circle {
                    transform: scale(1.2);
                }

                #fnmm-search-toggle:checked ~ .fnmm-header .search-icon .handle {
                    transform: rotate(45deg) scale(1.2);
                }

                .fnmm-icons .search-icon-wrapper label {
                    margin-right: 10px;
                    margin-bottom: 6px;
                }
            }
        ";

        // Hamburger icon styles
        $custom_css .= "
            .fnmm-header .fnmm-icons .hamburger-icon {
                display: flex;
                flex-direction: column;
                justify-content: space-between;
                width: 25px;
                height: 22px;
                cursor: pointer;
                transition: all 0.3s ease-in-out;
            }

            .fnmm-header .fnmm-icons .hamburger-icon .line {
                width: 100%;
                height: 4px;
                background-color: {$icon_color};
                border-radius: 5px;
                transition: all 0.3s ease-in-out;
            }

            #fnmm-menu-toggle:checked ~ .fnmm-header .fnmm-icons .hamburger-icon .line:nth-child(1) {
                transform: rotate(45deg) translate(5px, 5px);
            }

            #fnmm-menu-toggle:checked ~ .fnmm-header .fnmm-icons .hamburger-icon .line:nth-child(2) {
                opacity: 0;
            }

            #fnmm-menu-toggle:checked ~ .fnmm-header .fnmm-icons .hamburger-icon .line:nth-child(3) {
                transform: rotate(-45deg) translate(5px, -5px);
            }
        ";

        // Add search form styles
        $custom_css .= "
            #fly-nav-mobile-menu .fnmm-search-form {
                display: none;
                position: absolute;
                top: 100%;
                right: 20px;
                width: 90%;
                max-width: 300px;
                background-color: #fff;
                padding: 10px;
                box-shadow: 0 2px 5px rgba(0, 0, 0, 0.1);
                z-index: 9999;
                transform: translatex(0px) translatey(0px);
                border-radius: 5px;
                border-style: solid;
                border-width: 1px;
                border-color: rgba(0,0,0,0.07);
                margin-top: -10px;
            }

            #fnmm-search-toggle:checked ~ .fnmm-search-form {
                display: block;
            }
            .fnmm-search-form form {
                display: flex;
                width: 100%;
            }
            .fnmm-search-form input[type='search'],
            .fnmm-search-form label input[type=search] {
                flex-grow: 1;
                padding: 8px;
                font-size: 16px;
                transform: translatex(0px) translatey(0px);
                border: 1px solid rgba(189,195,199,0.52);
            }
            .fnmm-search-form button {
                padding: 8px 15px;
                background-color: {$icon_color};
                color: #fff;
                border: none;
                cursor: pointer;
                font-size: 16px;
            }
        ";

        // Remove underlines from menu items
        $custom_css .= "
            /* Remove underlines from menu items */
            #fly-nav-mobile-menu .fnmm-menu-items li a,
            #fly-nav-mobile-menu .fnmm-menu-items .sub-menu li a {
                text-decoration: none;
            }
        ";

        // Remove submenu toggle styles
        $custom_css .= "
            .fnmm-submenu-checkbox {
                display: none;
            }
            .fnmm-menu-items .menu-item-has-children {
                position: relative;
            }
            .fnmm-menu-items .menu-item-has-children > a {
                padding-right: 10px;
            }
            .fnmm-menu-items .sub-menu {
                display: none;
            }
            .fnmm-submenu-checkbox:checked ~ .sub-menu {
                display: block;
            }
        ";

        // Add submenu toggle styles
        $custom_css .= "
            .fnmm-submenu-toggle {
                display: none;
            }
            .fnmm-submenu-label {
                display: inline-block;
                width: 30px;
                height: 30px;
                position: absolute;
                right: 0;
                top: 5px;
                cursor: pointer;
                text-align: center;
                line-height: 30px;
            }
            .arrow-down-symbol {
                width: 0;
                height: 0;
                border-left: 8px solid transparent;
                border-right: 8px solid transparent;
                border-top: 9px solid var(--fnmm-menu-font-color);  /* Use the menu font color for the arrow */
                display: inline-block;
                transition: transform 0.3s ease;
            }
            .fnmm-menu-items .menu-item-has-children {
                position: relative;
            }
            .fnmm-menu-items .menu-item-has-children > a {
                padding-right: 40px;
            }
            .fnmm-menu-items .sub-menu {
                display: none;
            }
            .fnmm-submenu-toggle:checked ~ .sub-menu {
                display: block;
            }
            .fnmm-submenu-toggle:checked + .fnmm-submenu-label .arrow-down-symbol {
                transform: rotate(180deg);
            }
        ";

        // Add menu text color styles
        $custom_css .= "
            #fly-nav-mobile-menu .fnmm-menu-items,
            #fly-nav-mobile-menu .fnmm-menu-items a,
            #fly-nav-mobile-menu .fnmm-submenu-label {
                color: var(--fnmm-menu-font-color);
            }
        ";

        $custom_css .= "
            #fly-nav-mobile-menu .fnmm-search-form .search-form {
                display: flex;
                align-items: center;
            }
            #fly-nav-mobile-menu .fnmm-search-form .fnmm-search-field {
                flex-grow: 1;
                padding: 8px;
                border: 1px solid #ddd;
                border-right: none;
            }
            #fly-nav-mobile-menu .fnmm-search-form .fnmm-search-submit,
            #fly-nav-mobile-menu .fnmm-search-form .search-form .fnmm-search-submit[type='submit'] {
                padding: 8px 12px;
                background-color: {$icon_color};
                border: 1px solid #ddd;
                cursor: pointer;
                color: #fff;
                font-size: 14px;
                text-transform: uppercase;
            }
            #fly-nav-mobile-menu .fnmm-search-form .fnmm-search-submit:hover,
            #fly-nav-mobile-menu .fnmm-search-form .search-form .fnmm-search-submit[type='submit']:hover {
                opacity: 0.8;
            }
        ";

        // Add this new section to the custom CSS
        $custom_css .= "
            @media (max-width: 768px) {
                /* Ensure content starts after the mobile header */
                body {
                    padding-top: 60px; /* Adjust this value to match your header height */
                }

                #fly-nav-mobile-menu {
                    position: absolute; /* Keep it absolute */
                    top: 0;
                    left: 0;
                    width: 100%;
                    z-index: 9999;
                }

                /* Adjust other selectors if needed */
                .site-content,
                #content,
                .content-area,
                main {
                    position: relative; /* Ensure these are positioned relative to the body */
                    padding-top: 20px; /* Add some extra space between header and content */
                }

                /* Ensure the WordPress admin bar doesn't interfere */
                body.admin-bar #fly-nav-mobile-menu {
                    top: 46px; /* Height of the mobile admin bar */
                }

                body.admin-bar {
                    padding-top: 106px; /* 60px (header) + 46px (admin bar) */
                }
            }

            @media (max-width: 600px) {
                /* Adjust for WordPress mobile admin bar behavior */
                body.admin-bar #fly-nav-mobile-menu {
                    top: 0;
                }

                body.admin-bar {
                    padding-top: 60px; /* Just the header height */
                }
            }
        ";

        return $custom_css;
    }


}
